/*
 * main.c
 *
 * Created: 23N1223 22:41:39
 *  Author: Hiroyuki Yamada (Y-Logic)
 */ 

// * Fuse bit setting recommended *
// Set BODCFG = 0b00001110

#include <atmel_start.h>
#include <avr/pgmspace.h>
#include <utils/atomic.h>
#include <ccp.h>
#include <avr/sleep.h>
#include <avr/io.h>

#include "main.h"
#include "Calender.h"

// Fuse setting
FUSES = 
{
	.WDTCFG = FUSE_WDTCFG_DEFAULT,
	.BODCFG = 0b00001110,
	.OSCCFG = FUSE_OSCCFG_DEFAULT,
	.SYSCFG0 = FUSE_SYSCFG0_DEFAULT,
	.SYSCFG1 = FUSE_SYSCFG1_DEFAULT,
	.CODESIZE = FUSE_CODESIZE_DEFAULT,
	.BOOTSIZE = FUSE_BOOTSIZE_DEFAULT
};

const uint8_t BRIGHTNESS_LEVEL_PRESET_1[] = {36, 24, 16, 10}; // 5 is the most bright and larger value turns light down
const uint8_t BRIGHTNESS_LEVEL_PRESET_2[] = {105, 115, 128, 160}; // 128 is default, smaller value, darken led
uint8_t g_curBrightnessLevel = 0; // 3: brightest, 0: darkest
uint8_t g_brightnessSegment;
uint8_t g_brightnessDate;

// LED Segment Data
volatile uint8_t g_digitData[5];
// digit[0]: marks
//  bit 0: colon up
//  bit 1: colon down
//  bit 2: left-up dot of digit[4]

// Switch
volatile uint8_t g_swState;

// Display mode
volatile enum{DISP_TIME, DISP_SUN, DISP_LUNAR, DISP_DEMO, DISP_LP_DEBUG,
	DISP_ADJUST_YEAR, DISP_ADJUST_MONTH, DISP_ADJUST_DAY, 
	DISP_ADJUST_HOUR, DISP_ADJUST_MIN,
	} g_displayMode;


// Font Table
const uint8_t g_fontTable[] PROGMEM = {
	0b00111111, 0b00000110, 0b01011011, 0b01001111, 0b01100110,
	0b01101101, 0b01111101, 0b00100111, 0b01111111, 0b01101111,
	1,2,4,8,16,32,64,128
};
//#define GetFont(n) (pgm_read_byte(&(g_fontTable[n])));
static uint8_t GetFont(const uint8_t n) {
	return pgm_read_byte(&(g_fontTable[n]));
}


static void SREGSend(uint16_t d)
{
	SREG_LCK_set_level(false);
	uint8_t i;
	for(i=0; i<16; i++)
	{
		SREG_D_set_level((d & 0x8000)? true: false);
		SREG_DCK_set_level(true);
		SREG_DCK_set_level(false);
		d = d << 1;
	}
	SREG_LCK_set_level(true);
}
#define DYNAMIC_INTERVAL 500 // us
void UpdateLEDStatus()
{
	static uint8_t curDigit;
	
	if(curDigit == 0)
	{
		static uint8_t cnt;
		
		if(++cnt >= (100000 / 24 / DYNAMIC_INTERVAL))
		{
			cnt = 0;
			uint8_t pinState = 0;
			// check switch state
		
			PORTC_set_port_dir(0x0F, PORT_DIR_IN);
			_delay_us(100);
			pinState = PORTC_get_port_level();
		
			GetSwitchState(pinState, (uint8_t*)&g_swState);
		
			PORTC_set_port_dir(0x0F, PORT_DIR_OUT);
		}
	}
	// clear com level
	PORTC_set_port_level(0x0F, true);
	LED_COM0_set_level(true);
	
	// set segment
	PORTA_set_port_level(0xFF, false);
	if(curDigit <= 4)
	{
		uint8_t d = g_digitData[curDigit];
		PORTA_set_port_level(d, true);
	}
	
	// set com
	if(curDigit == 0)
	{
		LED_COM0_set_level(false);
	}
	else if(curDigit <= 4)
	{
		PORTC_set_port_level(1<<(curDigit-1), false);
	}
	
	curDigit ++;
	if(curDigit >= g_brightnessSegment) curDigit = 0;
}

///////////////////////////////////////
// Turn power state
///////////////////////////////////////
void ChangeClock32k(uint8_t enable)
{
	if(enable)
	{
		ccp_write_io((void*)&(CLKCTRL.MCLKCTRLA),
		CLKCTRL_CLKSEL_XOSC32K_gc /* External 32k oscillator */
		| 0 << CLKCTRL_CLKOUT_bp /* System clock out: disabled */);

	}
	else
	{
		ccp_write_io((void*)&(CLKCTRL.MCLKCTRLA),
		CLKCTRL_CLKSEL_OSCHF_gc /* Internal high-frequency oscillator */
			 | 0 << CLKCTRL_CLKOUT_bp /* System clock out: disabled */);
	}
}
// This function is called from pin interrupt. Because global interrupt is inhibited disable/enable control is not inserted.
void LowPower_Entry()
{
	
	// disable DC-DC converter
	uint8_t evout_val = EVSYS.USEREVSYSEVOUTD;
	EVSYS.USEREVSYSEVOUTD = EVSYS_USER_OFF_gc;
	
	// set all pin low
	SREGSend(0x00C0);
	uint8_t porta_level = PORTA_get_port_level();
	uint8_t portc_level = PORTC_get_port_level();
	uint8_t portd_level = PORTD_get_port_level();
	PORTA_set_port_level(0xFF, false);
	PORTD_set_port_level(0xFF, false);
	PORTC_set_port_dir(0x0F, false);
	
	// disable AC
	AC1.CTRLA &= ~0x01; // ENABLE = 0
	
	// disable TCA, TCB interrupt
	uint8_t tca_intval = TCA0.SINGLE.INTCTRL;
	TCA0.SINGLE.INTCTRL = 0;
	uint8_t tcb0_intval = TCB0.INTCTRL;
	TCB0.INTCTRL = 0;
	uint8_t tcb1_intval = TCB1.INTCTRL;
	TCB1.INTCTRL = 0;
	
	
	ChangeClock32k(1); // change main clock to to 32k
	
	//ENABLE_INTERRUPTS();
	
	// wait for power comes back
	while(1)
	{
		if(VSENSE_get_level() == true) break; // +5V detected
		sleep_cpu(); // enter to deep sleep
	}
	
	// (debug mode) stay main clock on and enable pull up
	
	/*
	_delay_ms(100);
	_delay_ms(100);
	_delay_ms(100);
	_delay_ms(100);
	_delay_ms(100);
	*/
	
	//DISABLE_INTERRUPTS();
	
	// restore
	ChangeClock32k(0); // change main clock to 4M
	// enable AC
	AC1.CTRLA |= 0x01; // ENABLE = 1
	VPORTA.OUT = porta_level;
	VPORTC.OUT = portc_level;
	VPORTD.OUT = portd_level;
	EVSYS.USEREVSYSEVOUTD = evout_val;
	
	TCA0.SINGLE.INTCTRL = tca_intval;
	TCB0.INTCTRL = tcb0_intval;
	TCB1.INTCTRL = tcb1_intval;
	
	//ENABLE_INTERRUPTS();
	
}

uint8_t g_multiNumericPhase; // phase
void ShowScrollNumerics(const uint8_t* displayDigits, uint8_t length, uint8_t loop)
{
	static uint8_t lastSec;
	if(lastSec != g_calender.subsec)
	{
		lastSec = g_calender.subsec;
		uint8_t phase = g_multiNumericPhase;
		g_digitData[1] = displayDigits[phase];
		g_digitData[2] = displayDigits[phase+1];
		g_digitData[3] = displayDigits[phase+2];
		g_digitData[4] = displayDigits[phase+3];
		
		if(phase+4 < length) g_multiNumericPhase++;
		else if(loop) g_multiNumericPhase = 0;
		
	}
}

void UpdateDisplay()
{
	// SREG
	uint16_t d = 0x00C0;
	if(g_displayMode == DISP_LUNAR) {
		d &= ~0x0040; // "lunar calender"
		//if(g_calender.mcMonth & 0x80) d &= ~0x0080; // "leap"
	}
	else if(g_displayMode == DISP_SUN) {
		d &= ~0x0080; // "sun calender"
	}
	d |= 0x0001 << ((g_calender.date6 + 4) % 6); // date6
	d |= 0x0100 << (g_calender.date7); // date7

	switch(g_displayMode)
	{
	case DISP_TIME:
	{
		g_digitData[0] = (g_digitData[0] & 0xFC) | ((g_calender.subsec & 1)? 0x00: 0x03);
		uint8_t h2 = g_calender.hour / 10;
		g_digitData[1] = h2? GetFont(h2): 0;
		g_digitData[2] = GetFont(g_calender.hour % 10);
		g_digitData[3] = GetFont(g_calender.min / 10);
		g_digitData[4] = GetFont(g_calender.min % 10);
	}
		break;
	case DISP_SUN:
	{
		uint8_t h2 = g_calender.month / 10;
		g_digitData[0] = 0;
		/*
		g_digitData[1] = h2? GetFont(h2): 0;
		g_digitData[2] = GetFont(g_calender.month % 10) | 0x80;
		g_digitData[3] = GetFont(g_calender.day / 10);
		g_digitData[4] = GetFont(g_calender.day % 10);
		*/
		
		uint8_t digits[16], cnt = 0;
		digits[cnt++] = 0;
		digits[cnt++] = 0;
		digits[cnt++] = 0;
		digits[cnt++] = GetFont(2);
		digits[cnt++] = GetFont(0);
		digits[cnt++] = GetFont(g_calender.year/10);
		digits[cnt++] = GetFont(g_calender.year%10) | 0x80;
		if(h2) digits[cnt++] = GetFont(h2);
		else digits[cnt++] = 0;
		digits[cnt++] = GetFont(g_calender.month % 10) | 0x80;
		digits[cnt++] = GetFont(g_calender.day / 10);
		digits[cnt++] = GetFont(g_calender.day % 10) | 0x80;
		
		ShowScrollNumerics(digits, cnt, 0);
		break;
	}
	case DISP_LUNAR:
	{
		uint8_t month = g_calender.mcMonth & 0x1F;
		uint8_t h2 = month / 10;
		g_digitData[0] = 0;
		/*
		g_digitData[1] = h2? GetFont(h2): 0;
		g_digitData[2] = GetFont((g_calender.mcMonth & 0x1F) % 10) | 0x80;
		g_digitData[3] = GetFont(g_calender.mcDay / 10);
		g_digitData[4] = GetFont(g_calender.mcDay % 10);
		*/
		
		uint8_t digits[16], cnt = 0;
		digits[cnt++] = 0;
		digits[cnt++] = 0;
		digits[cnt++] = 0;
		digits[cnt++] = GetFont(2);
		digits[cnt++] = GetFont(0);
		digits[cnt++] = GetFont(g_calender.mcYear/10);
		digits[cnt++] = GetFont(g_calender.mcYear%10) | 0x80;
		//if(g_calender.mcMonth & 0x80) digits[cnt++] = 0b00011100; // 'u'
		if(g_calender.mcMonth & 0x80) digits[cnt++] = 0b00111000; // 'L'
		if(h2) digits[cnt++] = GetFont(h2);
		else if(!(g_calender.mcMonth & 0x80)) digits[cnt++] = 0;
		digits[cnt++] = GetFont(month % 10) | 0x80;
		digits[cnt++] = GetFont(g_calender.mcDay / 10);
		digits[cnt++] = GetFont(g_calender.mcDay % 10) | 0x80;
		
		ShowScrollNumerics(digits, cnt, 0);
		break;
	}
	case DISP_ADJUST_YEAR:
		g_digitData[0] = 0;
		g_digitData[1] = GetFont(2);
		g_digitData[2] = GetFont(0);
		if(g_calender.subsec & 1) // blink
		{
			g_digitData[3] = GetFont(g_calender.year / 10);
			g_digitData[4] = GetFont(g_calender.year % 10);
		}
		else
		{
			g_digitData[3] = g_digitData[4] = 0;
		}
		break;
	case DISP_ADJUST_MONTH:
		g_digitData[0] = 0;
		if(g_calender.subsec & 1) // blink
		{
			g_digitData[1] = 0; g_digitData[2] = 0x80;
		}
		else
		{
			g_digitData[1] = GetFont(g_calender.month/10);
			g_digitData[2] = GetFont(g_calender.month % 10)| 0x80;
		}
		g_digitData[3] = GetFont(g_calender.day/10);
		g_digitData[4] = GetFont(g_calender.day % 10) | 0x80;
		break;
	case DISP_ADJUST_DAY:
		g_digitData[0] = 0;
		g_digitData[1] = GetFont(g_calender.month/10);
		g_digitData[2] = GetFont(g_calender.month % 10) | 0x80;
		if(g_calender.subsec & 1) // blink
		{
			g_digitData[3] = 0; g_digitData[4] = 0x80;
		}
		else
		{
			g_digitData[3] = GetFont(g_calender.day/10);
			g_digitData[4] = GetFont(g_calender.day % 10) | 0x80;
		}
		break;
	case DISP_ADJUST_HOUR:
		g_calender.sec = 0;
		g_digitData[0] = 0x03;
		if(g_calender.subsec & 1) // blink
		{
			g_digitData[1] = 0; g_digitData[2] = 0;
		}
		else
		{
			g_digitData[1] = GetFont(g_calender.hour/10);
			g_digitData[2] = GetFont(g_calender.hour % 10);
		}
		g_digitData[3] = GetFont(g_calender.min/10);
		g_digitData[4] = GetFont(g_calender.min % 10);
		break;
	
	case DISP_ADJUST_MIN:
		g_calender.sec = 0;
		g_digitData[0] = 0x03;
		g_digitData[1] = GetFont(g_calender.hour/10);
		g_digitData[2] = GetFont(g_calender.hour % 10);
		if(g_calender.subsec & 1) // blink
		{
			g_digitData[3] = 0; g_digitData[4] = 0;
		}
		else
		{
			g_digitData[3] = GetFont(g_calender.min/10);
			g_digitData[4] = GetFont(g_calender.min % 10);
		}
		break;
	default:
	{
		g_digitData[0] = 0x03;
		g_digitData[1] = GetFont(1);
		g_digitData[2] = GetFont(2);
		g_digitData[3] = GetFont(3);
		g_digitData[4] = GetFont(4);
		
		// demo display
		{
			uint8_t dd = g_calender.min * 60 + g_calender.sec;
			d = 0x00C0;
			d |= 0x0001 << (dd % 6); // date6
			d |= 0x0100 << (dd % 7); // date7
		}
		break;
	}
	}
	
	
	

	cli(); // cli/sei required, because when an interrupt occurs between IO port instructions,
	// the code does not work correctly and thus output signal will be wrong.
	// This occasion is NOT project specific, but the other projects potentially have the same kind of bug.
	SREGSend(d);
	sei();
}

void CheckSwitch()
{
	uint8_t swState = g_swState;
	g_swState = 0;
	if(g_displayMode >= DISP_ADJUST_YEAR && g_displayMode <= DISP_ADJUST_MIN)
	{
		switch(swState)
		{
		case SW_SW2: // -
		case SW_SW2 | SW_PUSHING: // -
			switch(g_displayMode)
			{
			case DISP_ADJUST_YEAR:
				if(g_calender.year > 23) g_calender.year --;
				break;
			case DISP_ADJUST_MONTH:
				if(g_calender.month == 1) g_calender.month = 12;
				else g_calender.month--;
				break;
			case DISP_ADJUST_DAY:
				if(g_calender.day == 1) g_calender.day = GetDayOfMonth(g_calender.year, g_calender.month);
				else g_calender.day --;
				break;
			case DISP_ADJUST_HOUR:
				if(g_calender.hour == 0) g_calender.hour = 23;
				else g_calender.hour --;
				break;
			case DISP_ADJUST_MIN:
				if(g_calender.min == 0) g_calender.min  = 59;
				else g_calender.min --;
				break;
			default:
				break;
			}
			GetCurMoonCalender(&g_calender, &g_calender.mcYear, &g_calender.mcMonth, &g_calender.mcDay);
			UpdateDate();
			break;
		case SW_SW3: // +
		case SW_SW3 | SW_PUSHING: // -
			switch(g_displayMode)
			{
			case DISP_ADJUST_YEAR:
				if(g_calender.year < 99) g_calender.year ++;
				break;
			case DISP_ADJUST_MONTH:
				g_calender.month++;
				if(g_calender.month > 12) g_calender.month = 1;
				break;
			case DISP_ADJUST_DAY:
				g_calender.day ++;
				if(g_calender.day > GetDayOfMonth(g_calender.year, g_calender.month)) g_calender.day = 1;
				break;
			case DISP_ADJUST_HOUR:
				g_calender.hour = (g_calender.hour+1) % 24;
				break;
			case DISP_ADJUST_MIN:
				g_calender.min = (g_calender.min+1) % 60;
				break;
			default:
				break;
			}
			GetCurMoonCalender(&g_calender, &g_calender.mcYear, &g_calender.mcMonth, &g_calender.mcDay);
			UpdateDate();
			break;
		case SW_SW1:
			if(g_displayMode == DISP_ADJUST_MIN)
			{
				g_calender.sec = 0;
				g_displayMode = DISP_TIME;
			}
			else
			{
				g_displayMode ++;
			}
			break;
		default:
			break;
		}
	}
	else
	{
		switch(swState)
		{
		case (SW_SW1 | SW_LONG):
			// Enter to time adjust
			if(g_displayMode < DISP_DEMO)
			{
				g_displayMode = DISP_ADJUST_YEAR;
			}
			break;
		case SW_SW3:
			// Change brightness level
			{
				g_curBrightnessLevel = (g_curBrightnessLevel+1) % 4;
				g_brightnessSegment = BRIGHTNESS_LEVEL_PRESET_1[g_curBrightnessLevel];
				g_brightnessDate = BRIGHTNESS_LEVEL_PRESET_2[g_curBrightnessLevel];
				AC1.DACREF = g_brightnessDate; // brightness ctrl
			}
			break;
		case SW_SW4:
			if(g_displayMode == DISP_DEMO)
			{
				g_displayMode = DISP_TIME;
			}
			else
			{
				g_displayMode ++;
			}
			g_multiNumericPhase = 0;
		default:
			break;
		}
	}
}
	

///////////////////////////////////////
// Main
///////////////////////////////////////

int main(void)
{
	g_brightnessSegment = BRIGHTNESS_LEVEL_PRESET_1[0];
	g_brightnessDate = BRIGHTNESS_LEVEL_PRESET_2[0];

	/* Initializes MCU, drivers and middleware */
	atmel_start_init();
	
	if(VSENSE_get_level() == false); // wait for +5V pin raise
	
	// Initialize PORTA as output (segment)
	PORTA_set_port_level(0xFF, false); // all output low
	PORTA_set_port_dir(0xFF, PORT_DIR_OUT);
	
	// Initialize PORTC as output (digit)
	PORTC_set_port_level(0x0F, false); // all output low
	PORTC_set_port_dir(0x0F, PORT_DIR_OUT);
	
	// Assign EVOUTD to PD7 from PD2 (code generated by START does not work)
	PORTMUX.EVSYSROUTEA |= 0b00001000; // EVOUTD = 1
	PWM_OUT_set_dir(PORT_DIR_OUT); // set as output for keeping direction during sleep
	
	// Comparator DAC
	AC1.DACREF = g_brightnessDate; // brightness ctrl
	
	SREGSend(0x0000);
	
	// Initialize calender
	g_calender.year = 24; // 2024
	g_calender.month = 1;
	g_calender.day = 20;
	GetCurMoonCalender(&g_calender, &g_calender.mcYear, &g_calender.mcMonth, &g_calender.mcDay);
	UpdateDate();
	
	g_digitData[0] = 0x03;
	g_digitData[1] = GetFont(0);
	g_digitData[2] = GetFont(1);
	g_digitData[3] = GetFont(2);
	g_digitData[4] = GetFont(3);

	sleep_enable();
	
	/* Replace with your application code */
	while (1)
	{
		_delay_ms(1);
		
		UpdateDisplay();
		
		CheckSwitch();
	}
}
